<?php

namespace Drupal\flow\Helpers;

use Drupal\flow\Flow;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Trait for subject plugins using the most recent entity from the Flow stack.
 */
trait StackSubjectTrait {

  use EntityRepositoryTrait;

  /**
   * The subject items from the Flow stack.
   *
   * @var \Drupal\Core\Entity\ContentEntityInterface[]
   */
  protected array $stackItems;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $task_mode = $configuration['task_mode'] ?? $instance->getBaseId();
    if (isset($task_mode)) {
      $stack = Flow::$stack[$task_mode] ?? [];
    }
    $instance->stackItems = !empty($stack) ? [end($stack)] : [];
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getSubjectItems(): iterable {
    $items = $this->stackItems;
    // Make sure to use the most recent state of the entities.
    $entity_repository = $this->getEntityRepository();
    foreach ($items as $i => $item) {
      if ($item->uuid() && ($loaded = $entity_repository->loadEntityByUuid($item->getEntityTypeId(), $item->uuid()))) {
        if ($item !== $loaded) {
          $items[$i] = $loaded;
        }
      }
    }
    return $items;
  }

}
